#ifndef SERVER_H
#define SERVER_H

#include <wayland-server.h>
#include <wlr/types/wlr_seat.h>

/* represents the server's state
 * important globals, shells, inputs, outputs
 */
struct half_server {
	struct wl_display* wl_display;
	struct wl_listener wl_display_destroy;

	struct wlr_backend* wlr_backend;
	struct wl_listener wlr_backend_destroy;
	struct wl_listener wlr_backend_new_input;
	struct wl_listener wlr_backend_new_output;

	struct wlr_compositor* wlr_compositor;
	struct wl_listener wlr_compositor_destroy;

	struct wlr_xdg_shell* wlr_xdg_shell;
	struct wl_listener wlr_xdg_shell_new_surface;
	struct wl_listener wlr_xdg_shell_destroy;

	struct wlr_data_device_manager* wlr_data_device_manager;
	struct wl_listener wlr_data_device_manager_destroy;

	struct wl_list seats; // of half_seat
	struct wl_list outputs; // of half_output

	const char* socket;
};

struct half_server* half_server_create();
void half_server_destroy(struct half_server*);

// sets up backends, sets WAYLAND_DISPLAY envvar, starts event loop
bool half_server_start(struct half_server*);

#endif
