#include "input/keyboard.h"

#include <stdio.h>
#include <stdlib.h>

#include <wlr/interfaces/wlr_input_device.h>
#include <xkbcommon/xkbcommon.h>

static void handle_wlr_input_device_destroy(struct wl_listener*, void* data);
static void handle_wlr_keyboard_modifiers(struct wl_listener*, void* data);
static void handle_wlr_keyboard_key(struct wl_listener*, void* data);

struct half_keyboard* half_keyboard_create(
		struct wlr_input_device* wlr_input_device) {
	struct half_keyboard* half_keyboard = calloc(1, sizeof(struct half_keyboard));
	if(!half_keyboard) {
		fprintf(stderr, "Failed to allocate half_keyboard\n");
		return NULL;
	}

	// init event handlers
	half_keyboard->wlr_input_device = wlr_input_device;
	half_keyboard->wlr_input_device_destroy.notify =
			handle_wlr_input_device_destroy;
	wl_signal_add(
			&half_keyboard->wlr_input_device->events.destroy,
			&half_keyboard->wlr_input_device_destroy);

	half_keyboard->wlr_keyboard = half_keyboard->wlr_input_device->keyboard;
	half_keyboard->wlr_keyboard_modifiers.notify = handle_wlr_keyboard_modifiers;
	wl_signal_add(
			&half_keyboard->wlr_keyboard->events.modifiers,
			&half_keyboard->wlr_keyboard_modifiers);
	half_keyboard->wlr_keyboard_key.notify = handle_wlr_keyboard_key;
	wl_signal_add(
			&half_keyboard->wlr_keyboard->events.key,
			&half_keyboard->wlr_keyboard_key);

	wl_signal_init(&half_keyboard->events.destroy);

	half_keyboard_reconfigure(half_keyboard);

	return half_keyboard;
}

void half_keyboard_destroy(struct half_keyboard* half_keyboard) {
	if(!half_keyboard) {
		return;
	}

	wl_signal_emit(&half_keyboard->events.destroy, half_keyboard);

	// unsubscribe from events (init is for safety, see WL docs)
	wl_list_init(&half_keyboard->wlr_input_device_destroy.link);
	wl_list_remove(&half_keyboard->wlr_input_device_destroy.link);

	wl_list_init(&half_keyboard->wlr_keyboard_modifiers.link);
	wl_list_remove(&half_keyboard->wlr_keyboard_modifiers.link);
	wl_list_init(&half_keyboard->wlr_keyboard_key.link);
	wl_list_remove(&half_keyboard->wlr_keyboard_key.link);

	wlr_input_device_destroy(half_keyboard->wlr_input_device);

	wl_list_init(&half_keyboard->link);
	wl_list_remove(&half_keyboard->link);

	free(half_keyboard);
}

bool half_keyboard_reconfigure(struct half_keyboard* half_keyboard) {
	struct xkb_context* xkb_context = xkb_context_new(XKB_CONTEXT_NO_FLAGS);
	if(!xkb_context) {
		fprintf(stderr, "Failed to create xkb_context\n");
		return false;
	}

	// use system default rules, in the future these could be configured TODO
	const struct xkb_rule_names xkb_rule_names = {
			.rules = NULL,
			.model = NULL,
			.layout = NULL,
			.variant = NULL,
			.options = NULL,
	};
	// compile keymap from rules
	struct xkb_keymap* xkb_keymap = xkb_keymap_new_from_names(
			xkb_context, &xkb_rule_names, XKB_KEYMAP_COMPILE_NO_FLAGS);
	if(!xkb_keymap) {
		fprintf(stderr, "Failed to compile xkb_keymap");
		xkb_context_unref(xkb_context);
		return false;
	}

	wlr_keyboard_set_keymap(half_keyboard->wlr_keyboard, xkb_keymap);

	// keymaps are reference counted
	xkb_keymap_unref(xkb_keymap);
	xkb_context_unref(xkb_context);

	// TODO these need configuring
	wlr_keyboard_set_repeat_info(half_keyboard->wlr_keyboard, 25, 600);

	return true;
}

static void handle_wlr_input_device_destroy(
		struct wl_listener* listener,
		void* data) {
	struct half_keyboard* half_keyboard =
			wl_container_of(listener, half_keyboard, wlr_input_device_destroy);
	(void)data;

	half_keyboard->wlr_input_device = NULL;
	half_keyboard_destroy(half_keyboard);
}

static void handle_wlr_keyboard_modifiers(
		struct wl_listener* listener,
		void* data) {
	(void)listener;
	(void)data;
}

static void handle_wlr_keyboard_key(
		struct wl_listener* listener,
		void* data) {
	(void)listener;
	(void)data;
}

/*
static void handle_key(struct wl_listener* listener, void* data) {
	struct half_keyboard* keyboard = wl_container_of(listener, keyboard, key);
	struct wlr_keyboard* wlr_keyboard = keyboard->device->keyboard;
	struct wlr_event_keyboard_key* event = data;

	// translate keycode from libinput -> xkb
	xkb_keycode_t keycode = event->keycode + 8;

	// extract (possibly multiple) keysyms from keycode based on keyboard's state
	const xkb_keysym_t* syms;
	int nsyms = xkb_state_key_get_syms(wlr_keyboard->xkb_state, keycode, &syms);

	// TEMP BODGE TO TERMINATE SERVER ON SUPER+SHIFT+G
	uint32_t modifiers = wlr_keyboard_get_modifiers(wlr_keyboard);
	for(int i = 0; i < nsyms; i++) {
		if(event->state == WLR_KEY_PRESSED && // event was key press
			 modifiers & (WLR_MODIFIER_LOGO | WLR_MODIFIER_SHIFT) &&
			 syms[i] == XKB_KEY_G) {
			wl_display_terminate(keyboard->server->wl_display);
		}
	}
}
*/
