#include "input/seat.h"

#include <stdio.h>
#include <stdlib.h>

#include <wlr/types/wlr_seat.h>

static void handle_wlr_seat_destroy(struct wl_listener*, void* data);
static void handle_half_keyboard_destroy(struct wl_listener*, void* data);

struct half_seat* half_seat_create(
		struct wl_display* wl_display,
		const char* name) {
	struct half_seat* half_seat = calloc(1, sizeof(struct half_seat));
	if(!half_seat) {
		fprintf(stderr, "Failed to allocate half_seat\n");
		return NULL;
	}

	// make the wlr_seat and register events
	half_seat->wlr_seat = wlr_seat_create(wl_display, name);
	if(!half_seat->wlr_seat) {
		fprintf(stderr, "Failed to create wlr_seat\n");
		half_seat_destroy(half_seat);
		return NULL;
	}
	half_seat->wlr_seat_destroy.notify = handle_wlr_seat_destroy;
	wl_signal_add(
			&half_seat->wlr_seat->events.destroy, &half_seat->wlr_seat_destroy);

	wl_list_init(&half_seat->keyboards);

	half_seat->half_keyboard_destroy.notify = handle_half_keyboard_destroy;

	return half_seat;
}

void half_seat_destroy(struct half_seat* half_seat) {
	if(!half_seat) {
		return;
	}

	wl_list_init(&half_seat->wlr_seat_destroy.link);
	wl_list_remove(&half_seat->wlr_seat_destroy.link);
	wl_list_init(&half_seat->half_keyboard_destroy.link);
	wl_list_remove(&half_seat->half_keyboard_destroy.link);

	wlr_seat_destroy(half_seat->wlr_seat);

	// destroy keyboards safely
	struct half_keyboard *pos, *tmp;
	wl_list_for_each_safe(pos, tmp, &half_seat->keyboards, link) {
		half_keyboard_destroy(pos);
	}

	wl_list_init(&half_seat->link);
	wl_list_remove(&half_seat->link);

	free(half_seat);
}

void half_seat_add_keyboard(
		struct half_seat* half_seat,
		struct half_keyboard* half_keyboard) {
	wl_list_insert(half_seat->keyboards.prev, &half_keyboard->link);
	wl_signal_add(
			&half_keyboard->events.destroy, &half_seat->half_keyboard_destroy);

	// set keyboard capability on seat
	half_seat->wlr_seat->capabilities |= WL_SEAT_CAPABILITY_KEYBOARD;
}

static void handle_wlr_seat_destroy(struct wl_listener* listener, void* data) {
	struct half_seat* half_seat =
			wl_container_of(listener, half_seat, wlr_seat_destroy);
	(void)data;

	half_seat->wlr_seat = NULL;
	half_seat_destroy(half_seat);
}

static void handle_half_keyboard_destroy(
		struct wl_listener* listener,
		void* data) {
	struct half_seat* half_seat =
			wl_container_of(listener, half_seat, half_keyboard_destroy);
	(void)data;

	// if last keyboard removed, disable capability
	if(wl_list_empty(&half_seat->keyboards)) {
		half_seat->wlr_seat->capabilities &= ~WL_SEAT_CAPABILITY_KEYBOARD;
	}
}
