#include "output.h"

#include <stdio.h>
#include <stdlib.h>

#include <wlr/types/wlr_output.h>

struct half_output* half_output_create() {
	struct half_output* half_output = calloc(1, sizeof(struct half_output));
	if(!half_output) {
		fprintf(stderr, "Failed to allocate half_output");
		return NULL;
	}

	half_output->wlr_output = NULL;

	return half_output;
}

void half_output_destroy(struct half_output* half_output) {
	if(!half_output) {
		return;
	}

	wl_list_init(&half_output->wlr_output_destroy.link);
	wl_list_remove(&half_output->wlr_output_destroy.link);

	wlr_output_destroy(half_output->wlr_output);

	wl_list_init(&half_output->link);
	wl_list_remove(&half_output->link);

	free(half_output);
}

void half_output_set_wlr_output(
		struct half_output* half_output,
		struct wlr_output* wlr_output) {
	// clean up old wlr_output (if it exists)
	if(half_output->wlr_output) {
		wl_list_init(&half_output->wlr_output_destroy.link);
		wl_list_remove(&half_output->wlr_output_destroy.link);
		wlr_output_destroy(half_output->wlr_output);
	}

	half_output->wlr_output = wlr_output;
}

void handle_wlr_output_destroy(struct wl_listener* listener, void* data) {
	struct half_output* half_output =
			wl_container_of(listener, half_output, wlr_output_destroy);
	(void)data;

	wl_list_init(&half_output->wlr_output_destroy.link);
	wl_list_remove(&half_output->wlr_output_destroy.link);
	half_output->wlr_output = NULL;

	// wlr_output dies but keep half_output alive in case wlr_output reconnects
}

/*
static void handle_frame(struct wl_listener* listener, void* data) {
	struct half_output* output = wl_container_of(listener, output, frame);
	struct wlr_output* wlr_output = data;
	struct wlr_renderer* renderer = wlr_backend_get_renderer(wlr_output->backend);

	wlr_output_attach_render(wlr_output, NULL); // TODO assert this

	// start renderer over output's entire surface
	wlr_renderer_begin(renderer, output->width, output->height);

	// clear to  white
	float colour[4] = {1.0, 1.0, 1.0, 1.0};
	wlr_renderer_clear(renderer, colour);

	wlr_renderer_end(renderer);
	wlr_output_commit(wlr_output);
}
*/
