#include "tree.h"

#include <stdio.h>
#include <stdlib.h>

struct half_tree_node* half_tree_branch_create(
		struct half_tree_node* parent,
		struct half_tree_node** self_handle,
		struct half_tree_node* left,
		struct half_tree_node* right) {
	struct half_tree_node* half_tree_node = calloc(1, sizeof(half_tree_node));
	if(!half_tree_node) {
		fprintf(stderr, "Failed to allocate half_tree_node\n");
		return NULL;
	}

	half_tree_node->half_tree_node_type = HALF_TREE_NODE_TYPE_BRANCH;

	half_tree_node->parent = parent;
	half_tree_node->self_handle = self_handle;

	// assign left and right and their parents and self_handles
	half_tree_node->left = left;
	left->parent = half_tree_node;
	*left->self_handle = half_tree_node->left;

	half_tree_node->right = right;
	right->parent = half_tree_node;
	*right->self_handle = half_tree_node->right;

	return half_tree_node;
}

struct half_tree_node* half_tree_leaf_create(
		struct half_tree_node* parent,
		struct half_tree_node** self_handle) {
	struct half_tree_node* half_tree_node = calloc(1, sizeof(half_tree_node));
	if(!half_tree_node) {
		fprintf(stderr, "Failed to allocate half_tree_node\n");
		return NULL;
	}

	half_tree_node->half_tree_node_type = HALF_TREE_NODE_TYPE_LEAF;

	half_tree_node->parent = parent;
	half_tree_node->self_handle = self_handle;

	return half_tree_node;
}

void half_tree_node_destroy(struct half_tree_node* half_tree_node) {
	if(!half_tree_node) {
		return;
	}

	free(half_tree_node);
}

void half_tree_destroy(struct half_tree_node* half_tree_node) {
	if(!half_tree_node) {
		return;
	}

	// destroy children as subtrees (tree shouldn't be huge so recursion fine)
	if(half_tree_node->half_tree_node_type == HALF_TREE_NODE_TYPE_BRANCH) {
		half_tree_destroy(half_tree_node->left);
		half_tree_destroy(half_tree_node->right);
	}

	half_tree_node_destroy(half_tree_node);
}

void half_tree_insert_node(
		struct half_tree_node* insertion_point,
		struct half_tree_node* half_tree_node) {
	struct half_tree_node** insert_point_handle = insertion_point->self_handle;
	/* swap insertion_point for a branch
	 * put insertion_point in left and new node in right
	 */
	*insert_point_handle = half_tree_branch_create(
			insertion_point->parent,
			insert_point_handle,
			insertion_point,
			half_tree_node);
}

void half_tree_remove_node(struct half_tree_node* half_tree_node) {
	if(half_tree_node->parent == NULL) {
		// removing the root, tree is now NULL
		*half_tree_node->self_handle = NULL;
	} else {
		// if we're removing left, right it sibling and vice versa
		struct half_tree_node* sibling =
				half_tree_node->parent->left == half_tree_node
						? half_tree_node->parent->right
						: half_tree_node->parent->left;

		// sibling replaces parent
		*sibling->parent->self_handle = sibling;
		sibling->self_handle = sibling->parent->self_handle;
		sibling->parent = sibling->parent->parent;

		half_tree_node_destroy(half_tree_node->parent);
	}

	half_tree_destroy(half_tree_node);
}
